#include "math.h"


//////// Integer square root ////////
// Jim Ulery's revision of Mark Borgerding's square root algorithm

Uint32 sqrt(Uint32 val)
{
	Uint32 temp,
		   g = 0,
		   b = 0x8000,
		   bshft = 15;

	do
	{
		if (val >= (temp = (((g << 1) + b) << bshft--)))
		{
			g += b;
			val -= temp;
		}
	} while (b >>= 1);

	return g;
}


//////// 1D math ////////

Uint32 distance(Uint32 a, Uint32 b)
{
	if (b < a)
		return a - b;
	else
		return b - a;
}

Sint32 distance(Sint32 a, Sint32 b)
{
	if (b < a)
		return a - b;
	else
		return b - a;
}


Sint32 sgn(Sint32 a)
{
	if (a > 0)
		return 1;
	else if (a < 0)
		return -1;
	else
		return 0;
}

#include <stdio.h>
char tolower(char c)
{
	if ((c >= 'A') && (c <= 'Z'))
	{
		c += 'a' - 'A';
	}

	return c;
}


Uint32 trim(Uint32 large, Uint32 little)
{
	if (large <= little)
		return 0;												// If the subtraction would produce a negative number (or 0 for speed), return 0.
	return (large - little);											// Else, do the subtraction
}

Uint16 trim(Uint16 large, Uint16 little)
{
	if (large <= little)
		return 0;												// If the subtraction would produce a negative number (or 0 for speed), return 0.
	return (large - little);											// Else, do the subtraction
}

Sint32 trim(Sint32 large, Sint32 little)
{
	if (large <= little)
		return 0;												// If the subtraction would produce a negative number (or 0 for speed), return 0.
	return (large - little);											// Else, do the subtraction
}

Sint16 trim(Sint16 large, Sint16 little)
{
	if (large <= little)
		return 0;												// If the subtraction would produce a negative number (or 0 for speed), return 0.
	return (large - little);											// Else, do the subtraction
}


Uint32 limit(Uint32 a, Uint32 max)
{
	if (a <= max)
		return a;
	return max;
}


//////// Quick swaping ////////

void swap(Uint32 &a, Uint32 &b)
{
	a ^= b;
	b ^= a;
	a ^= b;
}

void swap(Uint16 &a, Uint16 &b)
{
	a ^= b;
	b ^= a;
	a ^= b;
}


//////// Multiplication overflow ////////

Uint32 IMULHIDWORD(Uint32 A, Uint32 B)
{
	Uint32 HDW;

	__asm
	{
		mov		eax, A
		imul	B
		mov		HDW, edx
	}

	return HDW;
}


//////// Quick component separation ////////

void IDIVCOMP(Uint32 value, Uint32 width, Uint32 &x, Uint32 &y)
{
	Uint32 a, b;

	__asm
	{
		mov		eax, value
		cdq
		idiv	width
		mov		a, edx
		mov		b, eax
	}

	x = a;
	y = b;
}


//////// Quick memory copy ////////

void MEMCPY(void * src, void * dest, Uint32 len)
{
	__asm
	{
		mov ecx, len
		mov esi, src
		mov edi, dest
		cld
		rep movsb
	}
}


//////// 2D math ////////

Uint32 distance(Uint32 x1, Uint32 y1, Uint32 x2, Uint32 y2)
{
	return sqrt(distancesqr(x1, y1, x2, y2));
}

Uint32 distancesqr(Uint32 x1, Uint32 y1, Uint32 x2, Uint32 y2)
{
	Uint32 dx = distance(x1, x2);
	Uint32 dy = distance(y1, y2);

	return dx * dx + dy * dy;
}


//////// Quick rotation ////////

Uint32 ROL(Uint32 x, Uint32 r)
{	// Can be done with _emit and ROL/ROR opcodes
	r &= 31;

	return (x << r) | (x >> (32 - r));
}

Uint32 ROR(Uint32 x, Uint32 r)
{
	r &= 31;

	return (x >> r) | (x << (32 - r));
}

Uint32 ROT(Uint32 x, Sint32 r)
{
	if (sgn(r) == 1)
		return ROL(x, r);
	else
		return ROR(x, -r);
}
